@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2004, 2006, 2008  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004-2006  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1998-2003  Internet Software Consortium.
  *
- * Permission to use, copy, modify, and/or distribute this software for any
+ * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
@@ -15,7 +15,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: t_names.c,v 1.32.2.2.8.9 2008/01/17 23:45:27 tbox Exp $ */
+/* $Id: t_names.c,v 1.36.18.5 2006/12/07 23:57:58 marka Exp $ */
 
 #include <config.h>
 
@@ -38,7 +38,10 @@
 
 static char	*Tokens[MAXTOKS + 1];
 
-/*
+
+#ifdef	NEED_PBUF
+
+/*%
  * get a hex formatted dns message from a data
  * file into an isc_buffer_t
  * caller supplies data storage and the isc_buffer
@@ -46,8 +49,6 @@ static char	*Tokens[MAXTOKS + 1];
  * and return the data length
  */
 
-#ifdef	NEED_PBUF
-
 static char *
 ctoh(unsigned char c) {
 	int		val;
@@ -85,7 +86,7 @@ pbuf(isc_buffer_t *pbuf) {
 
 #endif	/* NEED_PBUF */
 
-/*
+/*%
  * Compare data at buf with data in hex representation at exp_data,
  * of length exp_data_len, for equality.
  * Return 0 if equal, else non-zero.
@@ -167,7 +168,7 @@ chkdata(unsigned char *buf, size_t buflen, char *exp_data,
 	return (result);
 }
 
-/*
+/*%
  * Get a hex formatted dns message from a data file into an isc_buffer_t.
  * Caller supplies data storage and the isc_buffer.  We read the file, convert,
  * setup the buffer and return the data length.
@@ -208,7 +209,6 @@ getmsg(char *datafile_name, unsigned char *buf, int buflen, isc_buffer_t *pbuf)
 		else if (('A' <= c) && (c <= 'Z'))
 			val = c - 'A'+ 10;
 		else {
-			(void)fclose(fp);
 			t_info("Bad format in datafile\n");
 			return (0);
 		}
@@ -222,7 +222,6 @@ getmsg(char *datafile_name, unsigned char *buf, int buflen, isc_buffer_t *pbuf)
 				/*
 				 * Buffer too small.
 				 */
-				(void)fclose(fp);
 				t_info("Buffer overflow error\n");
 				return (0);
 			}
@@ -258,14 +257,15 @@ bustline(char *line, char **toks) {
 	return (cnt);
 }
 
-/*
+
+#ifdef	NEED_HNAME_TO_TNAME
+
+/*%
  * convert a name from hex representation to text form
  * format of hex notation is:
  * %xXXXXXXXX
  */
 
-#ifdef	NEED_HNAME_TO_TNAME
-
 static int
 hname_to_tname(char *src, char *target, size_t len) {
 	int		i;
@@ -319,7 +319,7 @@ hname_to_tname(char *src, char *target, size_t len) {
 
 #endif	/* NEED_HNAME_TO_TNAME */
 
-/*
+/*%
  * initialize a dns_name_t from a text name, hiding all
  * buffer and other object initialization from the caller
  *
@@ -579,7 +579,7 @@ t_dns_name_isabsolute(void) {
 				 * label, bitpos, expected value.
 				 */
 				result = test_dns_name_isabsolute(Tokens[0],
-								atoi(Tokens[1])
+							        atoi(Tokens[1])
 								  == 0 ?
 								  ISC_FALSE :
 								  ISC_TRUE);
@@ -602,7 +602,7 @@ static const char *a8 =	"dns_name_hash(name, case_sensitive) returns "
 		"a hash of 'name' which is case_sensitive if case_sensitive "
 		"is true";
 
-/*
+/*%
  * a9 merged with a8.
  */
 
@@ -724,7 +724,7 @@ static const char *a10 =
 		"and sets nbitsp to the number of bits name1 and name2 "
 		"have in common";
 
-/*
+/*%
  * a11 thru a22 merged into a10.
  */
 static const char *
@@ -888,7 +888,7 @@ static const char *a23 =
 		"the relative ordering under the DNSSEC ordering relationship "
 		"of name1 and name2";
 
-/*
+/*%
  * a24 thru a29 merged into a23.
  */
 
@@ -991,7 +991,7 @@ static const char *a30 =
 		"about the relative ordering of name1 and name2 as if they "
 		"are part of rdata in DNSSEC canonical form";
 
-/*
+/*%
  * a31, a32 merged into a30.
  */
 
@@ -1094,7 +1094,7 @@ static const char *a33 =
 		"dns_name_issubdomain(name1, name2) returns true, "
 		"otherwise it returns false.";
 
-/*
+/*%
  * a34 merged into a33.
  */
 
@@ -1271,7 +1271,7 @@ static const char *a36 =
 		"dns_name_getlabel(name, n, labelp) initializes labelp "
 		"to point to the nth label in name";
 
-/*
+/*%
  * The strategy here is two take two dns names with a shared label in
  * different positions, get the two labels and compare them for equality.
  * If they don't match, dns_name_getlabel failed.
@@ -1364,7 +1364,7 @@ t_dns_name_getlabel(void) {
 				 */
 				result = test_dns_name_getlabel(Tokens[0],
 							      atoi(Tokens[1]),
-								   Tokens[2],
+							           Tokens[2],
 							      atoi(Tokens[3]));
 			} else {
 				t_info("bad format at line %d\n", line);
@@ -1386,7 +1386,7 @@ static const char *a37 =
 		"initializes target to point to the n label sequence of "
 		"labels in source starting with first";
 
-/*
+/*%
  * We adopt a similiar strategy to that used by the dns_name_getlabel test.
  */
 
